%********************************************************************
% baysda.m: Modal identification for SDOF system using the Bayesian *
%           spectral density approach                               *
%********************************************************************
clear
close all
clc
global Sy dis T dt wpos ES nlnpmin
%*** Parameters input ***
T=1000;                 % Total duration of measurement
dt=0.05;                % Sampling time step
wr=4;                   % Actual Frequency
zr=0.01;                % Actual Damping Ratio
Sf0r=1;                 % Actual Spectral Intensity of the Excitation
%************************

%  Data generation
disp('Data generation in progress ...')
Sy=spd_sda(wr,zr,Sf0r);  % Measurement of spectral density estimator

% Spectral density approach and optimization
semilogy([0:length(Sy)-1]/T,Sy)
xlabel('Frequency (Hz)')
ylabel('S_y_,_N')
disp('Please click at the peak of the spectrum')
[W,S]=ginput(1);
close; pause(1)
wpos=fix(1.2*W*T)+1; % ending index for frequency index set
Sy=Sy(2:wpos);
disp('Optimization in progress ...')
dis=1;nlnpmin=9e99;
para0=[wr zr Sf0r];
Npar=length(para0); % # parameters for identification
[para,nlnpmin]=fminsearch('psda',para0);

disp('Calculating Hessian')
dis=0;dx=para0/100;
for l1=1:Npar % Computation of hessian
    Hess(l1,l1)=(psda(para+[zeros(1,l1-1) dx(l1) zeros(1,Npar-l1)])-...
        2*nlnpmin+psda(para-[zeros(1,l1-1) dx(l1) zeros(1,Npar-l1)]))/dx(l1)^2;
    for l2=l1+1:Npar
        Hess(l1,l2)=(psda(para+[zeros(1,l1-1) dx(l1) zeros(1,l2-l1-1) dx(l2) zeros(1,Npar-l2)])-...
               psda(para+[zeros(1,l1-1) -dx(l1) zeros(1,l2-l1-1) dx(l2) zeros(1,Npar-l2)])-...
               psda(para+[zeros(1,l1-1) dx(l1) zeros(1,l2-l1-1) -dx(l2) zeros(1,Npar-l2)])+...
               psda(para-[zeros(1,l1-1) dx(l1) zeros(1,l2-l1-1) dx(l2) zeros(1,Npar-l2)]))/4/dx(l1)/dx(l2);
        Hess(l2,l1)=Hess(l1,l2);
    end
end
covm=inv(Hess);  % covariance matrix
SD=((diag(covm)).^0.5)';
results=[para0' para' SD' abs(SD./para)' abs((para0-para)./SD)'];
disp(['  --------------------------------------------------'])
disp(['    Actual  Identified   s.d.     c.o.v.     n.d.'])
disp(['  --------------------------------------------------'])
disp([results])
   
% Figures output
[P,S]=eig(covm(1:2,1:2));
Npl=40;
for p=1:Npl+1
    y(1,1)=(S(1,1))^0.5*cos(2*pi/Npl*p);
    y(2,1)=(S(2,2))^0.5*sin(2*pi/Npl*p);
    x(:,p)=P*y;
end
plot(para(1),para(2),'o')
hold on
plot(para0(1),para0(2),'x')
plot(para(1)+1.1774*x(1,:),para(2)+1.1774*x(2,:))
plot(para(1)+2.146*x(1,:),para(2)+2.146*x(2,:),'-.')
xlabel('\Omega (rad/s)')
ylabel('\zeta')
legend('Actual','Optimal Point','50%','90%')
grid
shg
figure
[P,S]=eig(covm(2:3,2:3));
for p=1:Npl+1
    y(1,1)=(S(1,1))^0.5*cos(2*pi/Npl*p);
    y(2,1)=(S(2,2))^0.5*sin(2*pi/Npl*p);
    x(:,p)=P*y;
end
plot(para(2),para(3),'o')
hold on
plot(para0(2),para0(3),'x')
plot(para(2)+1.1774*x(1,:),para(3)+1.1774*x(2,:))
plot(para(2)+2.146*x(1,:),para(3)+2.146*x(2,:),'-.')
xlabel('\zeta')
ylabel('S_f_0')
legend('Actual','Optimal Point','50%','90%')
grid
shg
clear x y P S A

Npl=30;
for p=1:Npl*2-1
    x(p)=para(1)+((p-1)/Npl-1)*4*(Hess(1,1))^(-0.5);
    y(p)=para(2)+((p-1)/Npl-1)*4*(Hess(2,2))^(-0.5);
    probfitx(p)=exp(-0.5*Hess(1,1)*(x(p)-para(1))^2);
    probfity(p)=exp(-0.5*Hess(2,2)*(y(p)-para(2))^2);
    probx(p)=exp(-psda([x(p) para(2:Npar)])+nlnpmin);
    proby(p)=exp(-psda([para(1) y(p) para(3:Npar)])+nlnpmin);
end
probx=probx/sum(probx)/(x(2)-x(1)); % normalization
proby=proby/sum(proby)/(y(2)-y(1));
probfitx=probfitx/sum(probfitx)/(x(2)-x(1));
probfity=probfity/sum(probfity)/(y(2)-y(1));

figure
subplot(211),plot(x,probx,'-',x,probfitx,'r-.')
xlabel('\Omega')
ylabel('PDF')
subplot(212),plot(y,proby,'-',y,probfity,'r-.')
xlabel('\zeta')
ylabel('PDF')