%**************************************************************
% nonsta.m: Bayesian time-domain approach using nonstationary *
%           response for SDOF linear systems                  *
%**************************************************************
close all
clear
clc
global dis T dt nlnpmin Yt Np
%*** Parameters input ***
T=50;                   % Total observed time
dt=0.05;                % Sampling interval
wr=5;                   % Actual frequency
zr=0.01;                % Actual Damping Ratio
Sf0r=10;                % Actual Spectral Intensity
nl=0.1;                 % Noise Level
%***********************

% Data generation
[Yt,sdn]=t_nonsta(T,dt,wr,zr,Sf0r,nl);

% Time Domain Analysis and optimization
dis=1;nlnpmin=1e99;
Sy=dt/2/pi*dt/T*abs(fft(Yt)).^2;Sy=Sy(1:fix(end/2));
semilogy([0:length(Sy)-1]/T,Sy)
xlabel('Frequency (Hz)')
ylabel('S_y_,_N')
disp('Please click at the peak of the spectrum')
[W,S]=ginput(1);
close; pause(1)
Np=max(fix(1/W/dt)+1,20); % not less than 20 point in any case
disp(['Np= ' num2str(Np)])
para0=[wr zr Sf0r sdn];
Npar=length(para0);       % no. of parameters
disp('Optimization in progress ...')
[para,nlnpmin]=fminsearch('pnonsta',para0);
dx=5e-4*ones(1,Npar);dis=0;
disp('Calculating Hessian')
for l1=1:Npar             % Computation of hessian
    Hess(l1,l1)=(pnonsta(para+[zeros(1,l1-1) dx(l1) zeros(1,Npar-l1)])-...
       2*nlnpmin+pnonsta(para-[zeros(1,l1-1) dx(l1) zeros(1,Npar-l1)]))/dx(l1)^2;
    for l2=l1+1:Npar
        Hess(l1,l2)=(pnonsta(para+[zeros(1,l1-1) dx(l1) zeros(1,l2-l1-1) dx(l2) zeros(1,Npar-l2)])-...
            pnonsta(para+[zeros(1,l1-1) -dx(l1) zeros(1,l2-l1-1) dx(l2) zeros(1,Npar-l2)])-...
            pnonsta(para+[zeros(1,l1-1) dx(l1) zeros(1,l2-l1-1) -dx(l2) zeros(1,Npar-l2)])+...
            pnonsta(para-[zeros(1,l1-1) dx(l1) zeros(1,l2-l1-1) dx(l2) zeros(1,Npar-l2)]))/4/dx(l1)/dx(l2);
        Hess(l2,l1)=Hess(l1,l2);  
    end
end
covm=inv(Hess); % covariance matrix
SD=((diag(covm)).^0.5)';
results=[para0;para;SD;abs(SD./para);abs(para-para0)./SD]';
disp(['  --------------------------------------------------'])
disp(['    Actual  Identified   s.d.     c.o.v.     n.d.'])
disp(['  --------------------------------------------------'])
disp([results])

% Drawing contours for in the frequency-damping ratio plane
[P,S]=eig(covm(1:2,1:2));
Npl=40;
for p=1:Npl+1
    y(1,1)=(S(1,1))^0.5*cos(2*pi/Npl*p);
    y(2,1)=(S(2,2))^0.5*sin(2*pi/Npl*p);
    x(:,p)=P*y;
end

figure(1)
plot(para0(1),para0(2),'x')
hold on
plot(para(1),para(2),'o')
plot(para(1)+1.1774*x(1,:),para(2)+1.1774*x(2,:))
plot(para(1)+2.146*x(1,:),para(2)+2.146*x(2,:),'-.')
xlabel('\Omega')
ylabel('\zeta')
legend('Actual','Optimal Point','50%','90%')
grid
clear x y P S A

% Drawing contours for in the damping ratio-spectral intensity plane
[P,S]=eig(covm(2:3,2:3));
for p=1:Npl+1
    y(1,1)=(S(1,1))^0.5*cos(2*pi/Npl*p);
    y(2,1)=(S(2,2))^0.5*sin(2*pi/Npl*p);
    x(:,p)=P*y;
end

figure(2)
plot(para0(2),para0(3),'x')
hold on
plot(para(2),para(3),'o')
plot(para(2)+1.1774*x(1,:),para(3)+1.1774*x(2,:))
plot(para(2)+2.146*x(1,:),para(3)+2.146*x(2,:),'-.')
xlabel('\zeta')
ylabel('S_f_0')
legend('Actual','Optimal Point','50%','90%')
grid
clear x y P S A 

% Drawing conditional PDFs
Npl=30;
for p=1:Npl*2-1
    x(p)=para(1)+((p-1)/Npl-1)*4*(Hess(1,1))^(-0.5);
    y(p)=para(2)+((p-1)/Npl-1)*4*(Hess(2,2))^(-0.5);
    probx(p)=exp(-pnonsta([x(p) para(2) para(3:Npar)])+nlnpmin);
    proby(p)=exp(-pnonsta([para(1) y(p) para(3:Npar)])+nlnpmin);
end
   
figure(3)
probx=probx/sum(probx)/(x(2)-x(1)); % normalization
proby=proby/sum(proby)/(y(2)-y(1));
subplot(211),plot(x,probx)
hold on
xlabel('\Omega')
ylabel('PDF')
subplot(212),plot(y,proby)
hold on
xlabel('\zeta')
ylabel('PDF')

% Gaussian approximation
for p=1:Npl*6-1
    x(p)=para(1)+((p-1)/Npl/3-1)*3.8*SD(1);
    y(p)=para(2)+((p-1)/Npl/3-1)*3.8*SD(2);
    probfitx(p)=exp(-0.5*([x(p) para(2) para(3:Npar)]-para)*Hess*([x(p) para(2) para(3:Npar)]-para)')*(det(Hess))^0.5/(2*pi)^2;
    probfity(p)=exp(-0.5*([para(1) y(p) para(3:Npar)]-para)*Hess*([para(1) y(p) para(3:Npar)]-para)')*(det(Hess))^0.5/(2*pi)^2;
end
probfitx=probfitx/sum(probfitx)/(x(2)-x(1)); % normalization
probfity=probfity/sum(probfity)/(y(2)-y(1));
subplot(211),plot(x,probfitx,'r--')
subplot(212),plot(y,probfity,'r--')