%************************************************************
% ms_mode.m: Model selection for modal ID with ambient data *
%            from shear building                            *
%************************************************************
clear
close all
clc
global ES dis T dt nlnpmin Sy Nd sensor Mmat Nm wpos
%****** Parameters input *****
T=60;                        % Duration of measurement
dt=1/100;                    % Sampling time interval
Nd=20;                       % # DOFs
Mmat=eye(Nd);                % Mass matrix
sensor=Nd;                   % observed DOF(s)
Kr=1500*ones(1,Nd);          % Actual stiffnesses
alpha=[0.044482 0.00168773]; % Rayleigh damping coeffients
Sf0r=1e-5;                   % Actual spectral intensity of excitation
nl=0.05;                     % Measurement noise level
mu0=pi*([1:Nd]*2-1);         % mean of prior PDF
sd0=mu0/10;                  % standard deviation of prior PDF
Nm1=5;Nm2=9;                 % Candidate model classes from Nm1-mode model 
                             % to Nm2-mode model
%*****************************

% Formation of C and K
Kmat=zeros(Nd,Nd);
Kmat(1,1)=Kr(1);
for l=2:Nd
    Kmat([l-1 l],[l-1 l])=Kmat([l-1 l],[l-1 l])+Kr(l)*[1 -1;-1 1];
end
Cmat=alpha(1)*Mmat+alpha(2)*Kmat;
[shape,freq]=eig(inv(Mmat)*Kmat);  % Eigenvalue problem
freq=diag(freq).^0.5;
[freq,I]=sort(freq);freq=freq';
shape=shape(:,I);
for m=1:Nd
    zeta(1,m)=shape(:,m)'*Cmat*shape(:,m)/2/freq(m);
    shape(:,m)=shape(:,m)/sign(shape(end,m));
end
% Computing the exact Rayleigh damping coefficients
a0=zeta(1)*2*freq(1)*freq(2)/sum(freq(1:2));
a1=zeta(1)*2/sum(freq(1:2));
for m=1:Nd % computing modal participating factor
    MPFr(m)=abs(sum(shape(:,m))*shape(end,m));
end

% Data Generation
[Sy,Sn0]=spd_ms_mode(Cmat,Kmat,Sf0r,nl);
semilogy([0:length(Sy)-1]/T,Sy)
xlabel('Frequency (Hz)')
ylabel('S_y_,_N')
disp('Please click upper limit frequency used for identification')
[W,S]=ginput(1);
close; pause(1)
wpos=max(fix(W*T)+1,100)  % ending index of the frequency index set

% Parametric Identification and Computation of Evidence
para=zeros(Nm2-Nm1+1,2*Nm2+3);
for Nm=Nm1:Nm2 % # modes considered
    nlnpmin=9e99;
    paratry=[freq(1:Nm) (Sf0r^0.5)*MPFr(1:Nm) a0 a1 Sn0];
    Npar=length(paratry);
    dis=0; % not to display the most updated parameters during optimization
    paratemp=fminsearch('pms_mode',paratry,optimset('MaxIter',1e5,'MaxFunEvals',1e5));
    [paratemp,nlnp0]=fminsearch('pms_mode',paratemp,optimset('MaxIter',1e5,'MaxFunEvals',1e5));
    disp(['Identified modal frequencies of the ' num2str(Nm) '-mode model (Hz):'])
    disp(num2str(round(paratemp(1:Nm)/2/pi*1e3)/1e3));
    lnp(Nm-Nm1+1)=-nlnp0; % max ln-likelihood
    para(Nm-Nm1+1,1:Npar)=paratemp;
    % computation of hessian & evidence
    dx=abs(paratry)/500;
    for l1=1:Npar
    Hess(l1,l1)=(pms_mode(paratemp+[zeros(1,l1-1) dx(l1) zeros(1,Npar-l1)])-...
    2*nlnp0+pms_mode(paratemp-[zeros(1,l1-1) dx(l1) zeros(1,Npar-l1)]))/dx(l1)^2;
       for l2=l1+1:Npar
       Hess(l1,l2)=(pms_mode(paratemp+[zeros(1,l1-1) dx(l1) zeros(1,l2-l1-1) dx(l2) zeros(1,Npar-l2)])-...
                pms_mode(paratemp+[zeros(1,l1-1) -dx(l1) zeros(1,l2-l1-1) dx(l2) zeros(1,Npar-l2)])-...
                pms_mode(paratemp+[zeros(1,l1-1) dx(l1) zeros(1,l2-l1-1) -dx(l2) zeros(1,Npar-l2)])+...
                pms_mode(paratemp-[zeros(1,l1-1) dx(l1) zeros(1,l2-l1-1) dx(l2) zeros(1,Npar-l2)]))/4/dx(l1)/dx(l2);
       Hess(l2,l1)=Hess(l1,l2);
       end
    end
    lnOc(Nm-Nm1+1)=-0.5*log(det(Hess))-sum(log(sd0(1:Nm))+((paratemp(1:Nm)-mu0(1:Nm)).^2)./sd0(1:Nm).^2);
end
results(1,:)=Nm1:Nm2;
results(2,:)=lnp;          % max ln-likelihood
results(3,:)=lnOc;         % ln-Ockham factor
lnEv=lnp+lnOc;
results(4,:)=lnEv;         % ln-evidence
P=exp(lnEv-max(lnEv));     % Pre-normalized Plausibility
results(5,:)=P/sum(P);     % Plausibility
disp(['---------------------------------------------------------------'])
disp(['#modes   max lnp*      ln Ockham     ln Evidence         P(Cj)'])
disp(['---------------------------------------------------------------'])
disp([num2str(round(results'*1e2)/1e2)])